==> LockingWorkflow-0_1.tgz: LockingWorkflow/Extensions/Install.py <==
##########################################################################
#                                                                        #
#           copyright (c) 2005 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""package installer"""

from StringIO import StringIO
from Products.LockingWorkflow.config import *
from Products.Archetypes.Extensions.utils import install_subskin

original_edit_actions = {}

def addRole(self, out, role):
    if role not in self.__ac_roles__:
        self._addRole(role)
        out.write("Added the '%s' role to the portal." % role)

def setupLockingWorkflow(self, out):
    wf_tool = self.portal_workflow
    wf_tool.manage_addWorkflow( id='locking_workflow',
                                workflow_type='locking_workflow ' + \
                                '(Two-state locking workflow)')
    out.write("Set up 'Locking Workflow'.\n")

def assignToTypes(self, out, types):
    wf_tool = self.portal_workflow
    for t in types:
        chain = wf_tool.getChainForPortalType(t)
        if 'locking_workflow' in chain:
            continue
        new_chain = ', '.join(chain) + ', locking_workflow'
        wf_tool.setChainForPortalTypes( (t,), new_chain)
        out.write("Assigned the locking workflow to '%s'.\n" % t)

def getAction(fti, action_id):
    for a in fti._actions:
        if a.getId() == action_id:
            return a
    return None

def changeEditActions(self, out, types):
    tt = self.portal_types
    for t in types:
        fti = tt[t]
        a = getAction(fti, 'edit')
        if a is None:
            continue
        edit_expression = a.getActionExpression()
        if edit_expression != 'lock_and_edit':
            original_edit_actions[t] = edit_expression
            a.setActionExpression('lock_and_edit')
            out.write("Changed the 'edit' action for '%s'.\n" % t)

def fixFCSettings(self, out, types):
    if not original_edit_actions:
        return None
    fct = self.portal_form_controller
    for ptype in original_edit_actions.keys():
        action = original_edit_actions[ptype]
        if 'base_edit' not in action:
            fct.addFormAction('lock_and_edit', 'success', ptype,
                              '', 'redirect_to', action )
    for ptype in types:
        fct.addFormAction('validate_integrity', 'success', ptype,
                          '', 'traverse_to', 'string:unlock_and_view' )
            
def addIndexToCatalog(self, out, idx):
    ct = self.portal_catalog
    if idx not in ct.indexes():
        ct.addIndex(idx, 'FieldIndex')
        out.write("Added a field index '%s' to the catalog.\n" % idx)


def install(self):
    out = StringIO()
    addRole(self, out, 'Editor')
    setupLockingWorkflow(self, out)
    assignToTypes(self, out, USE_FOR_TYPES)
    changeEditActions(self, out, USE_FOR_TYPES)
    fixFCSettings(self, out, USE_FOR_TYPES)
    install_subskin(self, out, GLOBALS)
    addIndexToCatalog(self, out, 'locking_state')
    out.write("Successfully set up the LockingWorkflow")
    return out.getvalue()

==> LockingWorkflow-0_2_0.tgz: LockingWorkflow/Extensions/Install.py <==
##########################################################################
#                                                                        #
#           copyright (c) 2005 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""package installer"""

from StringIO import StringIO
from Products.LockingWorkflow.config import *
from Products.Archetypes.Extensions.utils import install_subskin

original_edit_actions = {}

def addRole(self, out, role):
    if role not in self.__ac_roles__:
        self._addRole(role)
        out.write("Added the '%s' role to the portal.\n" % role)

def setupLockingWorkflow(self, out):
    wf_tool = self.portal_workflow
    wf_tool.manage_addWorkflow( id='locking_workflow',
                                workflow_type='locking_workflow ' + \
                                '(Two-state locking workflow)')
    out.write("Set up 'Locking Workflow'.\n")

def assignToTypes(self, out, types):
    wf_tool = self.portal_workflow
    for t in types:
        chain = wf_tool.getChainForPortalType(t)
        if 'locking_workflow' in chain:
            continue
        new_chain = ', '.join(chain) + ', locking_workflow'
        wf_tool.setChainForPortalTypes( (t,), new_chain)
        out.write("Assigned the locking workflow to '%s'.\n" % t)

def getAction(fti, action_id):
    for a in fti._actions:
        if a.getId() == action_id:
            return a
    return None

def changeEditActions(self, out, types):
    tt = self.portal_types
    for t in types:
        fti = tt[t]
        a = getAction(fti, 'edit')
        if a is None:
            continue
        edit_expression = a.getActionExpression()
        if edit_expression not in ['lock_and_edit',
                                   'string:${object_url}/lock_and_edit']:
            original_edit_actions[t] = edit_expression
            a.setActionExpression('lock_and_edit')
            out.write("Changed the 'edit' action for '%s'.\n" % t)

def fixFCSettings(self, out, types):
    if not original_edit_actions:
        return None
    fct = self.portal_form_controller
    for ptype in original_edit_actions.keys():
        action = original_edit_actions[ptype]
        if 'base_edit' not in action:
            fct.addFormAction('lock_and_edit', 'success', ptype,
                              '', 'redirect_to', action )
    for ptype in types:
        fct.addFormAction('validate_integrity', 'success', ptype,
                          '', 'traverse_to', 'string:unlock_and_view' )
        out.write("Made 'validate_integrity' traverse to " \
                  "'unlock_and_view' for '%s'.\n" \
                  % ptype)
        if PRIOR_TO_PLONE21:
            last_before_view = LAST_BEFORE_VIEW.get(ptype, None)
            if last_before_view is None:
                continue
            fct.addFormAction(last_before_view, 'success', ptype,
                              '', 'traverse_to', 'string:unlock_and_view' )
            out.write("Made '%s' traverse to 'unlock_and_view' for '%s'.\n" \
                      % (last_before_view, ptype))
            
def addIndexToCatalog(self, out, idx):
    ct = self.portal_catalog
    if idx not in ct.indexes():
        ct.addIndex(idx, 'FieldIndex')
        out.write("Added a field index '%s' to the catalog.\n" % idx)

def addPortlet(self, out):
    left_slots = list(self.getProperty('left_slots'))
    portlet =  'here/portlet_locked/macros/portlet'
    if portlet not in left_slots:
        left_slots.insert(0, portlet)
        self.manage_changeProperties(left_slots=left_slots)
        out.write("Inserted the 'locked' portlet into the top left slot.\n")


def install(self):
    out = StringIO()
    addRole(self, out, 'Editor')
    setupLockingWorkflow(self, out)
    assignToTypes(self, out, USE_FOR_TYPES)
    changeEditActions(self, out, USE_FOR_TYPES)
    fixFCSettings(self, out, USE_FOR_TYPES)
    install_subskin(self, out, GLOBALS)
    addIndexToCatalog(self, out, 'locking_state')
    if ADD_PORTLET:
        addPortlet(self, out)
    out.write("Successfully set up the LockingWorkflow")
    return out.getvalue()
